#!/usr/bin/env python2
#
# Copyright (c) Members of the EGEE Collaboration. 2004-2010.
# See http://www.eu-egee.org/partners/ for details on the
# copyright holders.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# MySQL database initialisation file - setups the Apel database to enable 
# access for the defined user.
#

import os
import socket
import string
import sys

class DBSetup:
    
    def __init__(self, args):
        
        self.user = None
        self.password = None
        self.dbname = None
        self.outfile = None
        self.gkhost = None
        self.cehost = None
        self.schemaFile = None
        
        if len(args) == 0:
            self.__printHelp()
        previous = ""
        for arg in args:
            if previous == "-o":
                self.outfile = arg.strip()
            elif previous == "-u" or previous == "--user":
                self.user = arg.strip()
            elif previous == "-p" or previous == "--password":
                self.password = arg.strip()
            elif previous == "-d" or previous == "--database":
                self.dbname = arg.strip()
            elif arg == "-h" or arg == "--help":
                self.__printHelp()
            elif previous == "-g" or previous == "--gkhost":
                self.gkhost = arg.strip()
            elif previous == "-c" or previous == "--cehost":
                self.cehost = arg.strip()
            previous = arg
        
        try:
            apelhome = os.environ['APEL_HOME']
        except KeyError:
            print "ERROR: Environment variable APEL_HOME must be set"
            sys.exit(1)
        
        self.schema = os.path.join(apelhome, 
            'share/glite-apel-core/scripts/apel-schema.sql')
        self.__checkParams()
        self.__createScript()
        print
        print 'SQL script successfully created'
        print
        print 'To create the database and setup user permissions, ' + \
            'you need to run the following:'
        print
        print '\tmysql -u root -p < %s' % (self.outfile)
        print
        print 'and enter the mysql root password when prompted'
        print 
       
    def __checkParams(self):
        
        """ Checks the user has defined all input paramters. """
        
        error = False
        if self.user == None:
            print "ERROR: No database user name defined"
            error = True
        if self.password == None:
            print "ERROR: No database password defined"
            error = True
        if self.outfile == None:
            print "ERROR: No output file defined"
            error = True
        if self.gkhost == None:
            print "ERROR: No GK hostname defined"
            error = True
        if self.cehost == None:
            print "ERROR: No CE hostname defined"
            error = True
        if self.dbname == None:
            print "ERROR: No database name defined"
            error = True
        if error:
            sys.exit(1)
        
    def __printHelp(self):
        
        script = 'apel-db-config.py'
        print
        print "Creates an SQL script that installs a user, " + \
            "database and schema for the Apel application (using MySQL). "
        print
        print "Usage: " + script + " [options]"
        print ""
        print "Options:"
        print
        print "    -h, --help                Display help information"
        print "    -d, --database <db name>  Name of the database to be " + \
            "created"
        print "    -u, --user <user>         Username that can access the " + \
            "created database"
        print "    -p, --password <password> Username's password to access " + \
            "the database password"
        print "    -g, --gkhost <hostname>   Full hostname of the " + \
            "gatekeeper"
        print "    -c, --cehost <hostname>   Full hostname of the " + \
            "computing element"
        print "    -o <file>                 Output file where the SQL " + \
            "will be written"
        print
        print "Example:"
        print
        print  script + " --user tuti --password fruiti --database " + \
            "accounting --gkhost gk.host --cehost ce.host -o ./accounting.sql"
        print
        sys.exit(0)
        
    def __createScript(self):
            
        hostname = socket.gethostname()
        f = file(self.outfile, 'w')
        f.write("USE mysql\n")
        f.write("GRANT SELECT, INSERT, UPDATE, DELETE ON " + 
            "%s.* TO '%s' IDENTIFIED BY '%s';\n" % \
            (self.dbname, self.user, self.password))
        f.write("DELETE FROM user WHERE user LIKE '%s';\n" % self.user)
        f.write("DELETE FROM db WHERE user LIKE '%s';\n" % self.user)
        f.write("INSERT INTO user (host,user,password,create_priv,drop_priv) " +
            "VALUES ('localhost','%s',password('%s'),'Y','Y');\n" % \
            (self.user, self.password))
        f.write("INSERT INTO user (host,user,password,create_priv,drop_priv) " +
            "VALUES ('localhost.localdomain','%s',password('%s'),'Y','Y');\n" % \
            (self.user, self.password))
        f.write("INSERT INTO user (host,user,password,create_priv,drop_priv) " +
            "VALUES ('%s','%s',password('%s'),'Y','Y');\n" % \
            (hostname, self.user, self.password))
        if self.cehost != hostname:
            f.write("INSERT INTO user (host,user,password,create_priv, " +
                "drop_priv) VALUES ('%s','%s',password('%s'),'Y','Y');\n" % \
                (self.cehost, self.user, self.password))
        if self.gkhost != hostname and self.gkhost != self.cehost:
            f.write("INSERT INTO user (host,user,password,create_priv, " + 
                "drop_priv) VALUES ('%s','%s',password('%s'),'Y','Y');\n" % \
                (self.gkhost, self.user, self.password))
        f.write("INSERT INTO db (host,db,user,select_priv,insert_priv," +
            "update_priv,delete_priv,create_priv, alter_priv, " + 
            "Create_tmp_table_priv) VALUES " +
            "('%%','%s','%s','Y','Y','Y','Y','Y','Y','Y');\n" % \
            (self.dbname, self.user))
        f.write("FLUSH PRIVILEGES;\n")
        f.write("DROP DATABASE IF EXISTS %s;\n" % self.dbname)
        f.write("CREATE DATABASE %s;\n" % self.dbname)
        f.write("USE %s;\n" % self.dbname)
        f.write(self.__getSchema())
        f.close()
        
    def __getSchema(self):
        
        f = open(self.schema)
        data = f.read();
        f.close()
        return data
        
if __name__ == "__main__":
    
    DBSetup(sys.argv[1:])
    
